/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "AboutViewController.h"
#import "AboutTableViewController.h"
#import "AboutWebViewController.h"

@interface AboutViewController () <AboutTabelSectionDelegate>

@property (nonatomic, weak) AboutTableViewController *aboutTableViewController;
@property (nonatomic, weak) IBOutlet UITextView *aboutTextView;

@end

@implementation AboutViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self buildHeader:@"About" and:@""];
    self.aboutTableViewController = [[self childViewControllers] firstObject];
    self.aboutTableViewController.aboutTableSectionDelegate = self;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (void)didSelectAboutSection:(TTAboutSection)aboutSection {
    [self performSegueWithIdentifier:@"PerformAboutWebViewController" sender:@(aboutSection)];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if ([segue.destinationViewController isKindOfClass:[AboutWebViewController class]]) {
        AboutWebViewController *aboutWebViewController = (AboutWebViewController *)segue.destinationViewController;
        TTAboutSection aboutSection = (TTAboutSection)[sender integerValue];
        aboutWebViewController.htmlString = [[NSString alloc] initWithData:[self attributionHTML:aboutSection] encoding:NSUTF8StringEncoding];
    }
}

- (NSData *)attributionHTML:(TTAboutSection)aboutSection {
    NSString *resourceFramework = @"/Frameworks/FRAMEWORK_NAME.framework/ATTRIBUTION.html";
    NSString *resourcePathFramework;
    switch (aboutSection) {
        case TTAboutSectionMaps:
            resourcePathFramework = [resourceFramework stringByReplacingOccurrencesOfString:@"FRAMEWORK_NAME" withString:@"TomTomOnlineSDKMaps"];
            break;

        case TTAboutSectionSearch:
            resourcePathFramework = [resourceFramework stringByReplacingOccurrencesOfString:@"FRAMEWORK_NAME" withString:@"TomTomOnlineSDKSearch"];
            break;

        case TTAboutSectionRouting:
            resourcePathFramework = [resourceFramework stringByReplacingOccurrencesOfString:@"FRAMEWORK_NAME" withString:@"TomTomOnlineSDKRouting"];
            break;

        case TTAboutSectionUtils:
            resourcePathFramework = [resourceFramework stringByReplacingOccurrencesOfString:@"FRAMEWORK_NAME" withString:@"TomTomOnlineUtils"];
            break;

        case TTAboutSectionExamples:
            resourcePathFramework = @"/ATTRIBUTION.html";

        default:
            break;
    }

    resourcePathFramework = [NSString stringWithFormat:@"%@%@", [[NSBundle mainBundle] resourcePath], resourcePathFramework];

    NSData *data = [NSData dataWithContentsOfFile:resourcePathFramework];
    return data;
}

@end
