/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "AdditionalDataViewController.h"
#import <TomTomOnlineSDKMaps/TomTomOnlineSDKMaps.h>
#import <TomTomOnlineSDKSearch/TomTomOnlineSDKSearch.h>
#import "Locations.h"
#import "PolygonAdditionalDataVisitior.h"

@interface AdditionalDataViewController () <UIExampleViewControllerDelegate>

@property (nonatomic, weak) IBOutlet TTMapView *mapView;
@property (nonatomic, strong) TTSearch *search;

@property (nonatomic, strong) TTPolygon *polygon;

@end

@implementation AdditionalDataViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self buildHeader:@"Additional data" and:@"Polygons for geographies"];
    [self createSegmentedControllWithTag:-1 options:@[@"Amsterdam", @"Berlin", @"Poland"] multiSelect:NO select:-1];
    [self.mapView centerOnCoordinate:COORDINATE_AMSTERDAM withZoom:10];
    self.delegate = self;
}

- (void)perfromSearch:(NSString *)value {
    TTSearchQuery *query = [[[TTSearchQueryBuilder createWithTerm:value] withIdxSet:[NSSet setWithObject:@(TTSearchIndexGeographies)]] build];
    [[TTSearch new] searchWithQuery:query completionHandle:^(TTSearchResponse * _Nullable response, TTResponseError * _Nullable error) {
        TTAdditionalDataSources *ads = response.results.firstObject.additionalDataSources;
        TTGeometryDataSource *gds = ads.geometryDataSource;
        TTAdditionalDataSearchQuery *query = [[TTAdditionalDataSearchQueryBuilder createWithDataSource:gds] build];
        [[TTAdditionalDataSearch new] additionalDataSearchWithQuery:query completionHandler:^(TTAdditionalDataSearchResponse * _Nullable result, TTResponseError * _Nullable error) {
            PolygonAdditionalDataVisitior *visitior = [PolygonAdditionalDataVisitior new];
            [[[result results] firstObject] visitResult:visitior];
            [self drawingPolygonAdditionalDataVisitior:visitior];
        }];
    }];
}

- (void)drawingPolygonAdditionalDataVisitior:(PolygonAdditionalDataVisitior *)visitior {
    if (visitior.lineStrings) {

        NSMutableArray<TTPolygon *> *polygons = [NSMutableArray new];
        for (TTADPLineString *lineString in visitior.lineStrings) {
            TTPolygon *polygon = [TTPolygon polygonWithCoordinatesData:lineString opacity:0.7 color:[UIColor redColor] colorOutline:[UIColor redColor]];
            [polygons addObject:polygon];
            [self.mapView.annotationManager addOverlay:polygon];
        }

        [self.mapView zoomToCoordinatesDataCollection:polygons];
        
    }
}

- (void)optionButtonTag:(NSInteger)tag index:(int)index {
    [self.mapView.annotationManager removeAllOverlays];
    NSString *term;
    switch (index) {
        case 0:
            term = @"Amsterdam";
            break;
        case 1:
            term = @"Berlin";
            break;
        case 2:
            term = @"Poland";
            break;
        default:
            break;
    }
    [self perfromSearch:term];
}

@end
