/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "BatchSearchViewController.h"
#import <TomTomOnlineSDKMaps/TomTomOnlineSDKMaps.h>
#import <TomTomOnlineSDKSearch/TomTomOnlineSDKSearch.h>
#import "Locations.h"
#import "Colors.h"

#define GEOMETRY_RADIUS 4000

@interface BatchSearchViewController() <UIExampleViewControllerDelegate, TTBatchSearchDelegate, TTBatchVisistor>

@property (nonatomic, weak) IBOutlet TTMapView *mapView;
@property (nonatomic, strong) NSArray *options;

@property (nonatomic, strong) TTBatchSearch *batch;

@end

@implementation BatchSearchViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.delegate = self;
    self.options = @[@"Parking",@"Gas", @"Bar"];
    [self buildHeader:@"Batch search" and:@"Search for categories in different locations"];
    [self createSegmentedControllWithTag:-1 options:self.options multiSelect:NO select:-1];
    [self.mapView centerOnCoordinate:COORDINATE_AMSTERDAM_CENTER_LOCATION withZoom:10];
    self.mapView.annotationManager.clustering = YES;
    
    self.batch = [[TTBatchSearch alloc] init];
    self.batch.delegate = self;
}

- (void)optionButtonTag:(NSInteger)tag index:(int)index {
    [self.mapView.annotationManager removeAllAnnotations];
    [self.mapView.annotationManager removeAllOverlays];
    [self performSearch:self.options[index]];
}

- (void)performSearch:(NSString *)term {
    TTSearchQuery *searchQuery1 = [[[[TTSearchQueryBuilder createWithTerm:term]
                                    withPosition:COORDINATE_AMSTERDAM_CENTER_LOCATION]
                                   withLimit:@(10)]
                                  build];
    TTSearchQuery *searchQuery2 = [[[[TTSearchQueryBuilder createWithTerm:term]
                                     withPosition:COORDINATE_HAARLEM]
                                    withLimit:@(15)]
                                   build];
    TTSearchCircle *geometry = [TTSearchCircle circleWithCenter:COORDINATE_HOOFDDORP radius:GEOMETRY_RADIUS];
    TTGeometrySearchQuery *geometryQuery = [[TTGeometrySearchQueryBuilder createWithTerm:term
                                                                            searchShapes:[NSArray arrayWithObject:geometry]]
                                            build];
    
    TTBatchQuery *batchQuery = [[[[TTBatchQueryBuilder createSearchQuery:searchQuery1]
                                  addSearchQuery:searchQuery2]
                                 addGeometryQuery:geometryQuery]
                                build];
 
    [self.batch batchSearchWithQuery:batchQuery];
}

#pragma mark TTBatchSearchDelegate

- (void)batch:(TTBatchSearch* _Nonnull)batch completedWithResponse:(TTBatchResponse* _Nonnull)response{
    [response visit:self];
}

- (void)batch:(TTBatchSearch* _Nonnull)batch failedWithError:(TTResponseError* _Nonnull)error {
    
}

#pragma mark TTBatchVisistor

- (void)visitSearch:(TTSearchResponse *)response {
    for (TTSearchResult *result in response.results) {
        TTAnnotation *annotation = [TTAnnotation annotationWithCoordinate:result.position];
        [self.mapView.annotationManager addAnnotation:annotation];
    }
}

- (void)visitGeometrySearch:(TTGeometrySearchResponse *)response {
    for (TTGeometrySearchResult *result in response.results) {
        TTAnnotation *annotation = [TTAnnotation annotationWithCoordinate:result.position];
        [self.mapView.annotationManager addAnnotation:annotation];
    }
    
    TTCircle *circle = [TTCircle circleWithCenterCoordinate:COORDINATE_HOOFDDORP radius:GEOMETRY_RADIUS width:2 color:TTColorOverlay fill:true colorOutlet:TTColorOverlayOutline];
    [self.mapView.annotationManager addOverlay:circle];
    
    [self.mapView zoomToAllAnnotations];
}

@end
