/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "EntryPointsViewController.h"
#import <TomTomOnlineSDKMaps/TomTomOnlineSDKMaps.h>
#import <TomTomOnlineSDKSearch/TomTomOnlineSDKSearch.h>
#import "Locations.h"

@interface EntryPointsViewController () <TTMapViewDelegate, UIExampleViewControllerDelegate, TTAnnotationDelegate>

@property (nonatomic, weak) IBOutlet TTMapView *mapView;
@property (nonatomic, strong) TTSearch *search;

@end

@implementation EntryPointsViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self buildHeader:@"Entry Points" and:@"Amsterdam"];
    [self createSegmentedControllWithTag:-1 options:@[@"Airport", @"Shopping Mall"] multiSelect:NO select:-1];
    self.mapView.delegate = self;
    self.mapView.annotationManager.delegate = self;
    self.delegate = self;
    [self.mapView centerOnCoordinate:COORDINATE_AMSTERDAM withZoom:10];
}

- (void)performSearchForTerm:(NSString *)term {
    [self.search cancel];

    TTSearch *search = [[TTSearch alloc] init];
    TTSearchQuery *query = [[TTSearchQueryBuilder createWithTerm:term] build];
    [search searchWithQuery:query completionHandle:^(TTSearchResponse * _Nullable response, TTResponseError * _Nullable error) {
        [self displayResponse:response.results.firstObject];
    }];
}

- (void)displayResponse:(TTSearchResult *)searchResult {
    NSMutableArray <TTAnnotation *> *annotations = [NSMutableArray new];

    // Add POI
    TTAnnotation *annotationPoi = [TTAnnotation annotationWithCoordinate:searchResult.position
                                                                   image:[TTAnnotation defaultImage]
                                                                     tag:searchResult.poi.name
                                                                  anchor:TTAnnotationAnchorBottom
                                                                    type:TTAnnotationTypeFocal];
    [self.mapView.annotationManager addAnnotation:annotationPoi];
    [annotations addObject:annotationPoi];

    for (TTEntryPoint *entryPoint in searchResult.entryPoints) {
        CLLocationCoordinate2D coordinate = entryPoint.position;
        UIImage *image = [UIImage imageNamed:@"entry_point"];
        NSString *tag = [NSString stringWithFormat:@"Entry point type: %@", entryPoint.type];
        TTAnnotation *annotation = [TTAnnotation annotationWithCoordinate:coordinate
                                                                    image:image
                                                                      tag:tag
                                                                   anchor:TTAnnotationAnchorBottom
                                                                     type:TTAnnotationTypeFocal];
        [self.mapView.annotationManager addAnnotation:annotation];
        [annotations addObject:annotation];
    }
    [self.mapView zoomToAnnotations:annotations];

}

- (UIView<TTCalloutView> *_Nonnull)annotationManager:(id<TTAnnotationManager>)annotationManager viewForSelectedAnnotation:(TTAnnotation *)selectedAnnotation {
    return [[TTCalloutViewSimple alloc] initWithText:selectedAnnotation.annotationTag];
}

- (void)optionButtonTag:(NSInteger)tag index:(int)index {
    NSString *term;
    switch (index) {
        case 0:
            term = @"Amsterdam Airport Schiphol";
            break;
        case 1:
            term = @"Kalvertoren Singel 451";
            break;
        default:
            break;
    }
    [self performSearchForTerm:term];
}

@end
