/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "GeometrySearchViewController.h"
#import <TomTomOnlineSDKSearch/TomTomOnlineSDKSearch.h>
#import <TomTomOnlineSDKMapsUIExtensions/TomTomOnlineSDKMapsUIExtensions.h>
#import "Colors.h"
#import "Locations.h"

@interface GeometrySearchViewController () <TTMapViewDelegate, UIExampleViewControllerDelegate>

@property (nonatomic, weak) IBOutlet TTMapView *mapView;
@property (weak, nonatomic) IBOutlet TTControlView *controlView;

@property (nonatomic, strong) TTCircle *mapCircle;
@property (nonatomic, strong) TTPolygon *mapPolygon;
@property (nonatomic, strong) NSMutableArray<TTSearchShape *>* shapes;

@property (nonatomic, strong) TTGeometrySearch *geometrySearch;

@end

@implementation GeometrySearchViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self buildHeader:@"Geometry search" and:@"Amsterdam"];
    [self createSegmentedControllWithTag:-1 options:@[@"Parking", @"ATM", @"Grocery"] multiSelect:NO select:-1];
    self.delegate = self;
    self.mapView.delegate = self;
    [self.mapView centerOnCoordinate:CLLocationCoordinate2DMake(52.3691851,4.8505632) withZoom:10];
    self.controlView.mapView = self.mapView;
    [self.controlView initDefaultCenterButton];
    [self.controlView initDefaultCompassButton];
    self.controlView.topLayoutConstraintCompassButton.constant = TopLayoutConstraintCompass;
    self.controlView.bottomLayoutConstraintCenterButton.constant = BottomLayoutConstraintCenter;
    [self createShapesForSearch];
}

- (void)performSearchForTerm:(NSString *)term {
    [self.geometrySearch cancel];

    TTGeometrySearch *search = [[TTGeometrySearch alloc] init];
    TTGeometrySearchQuery *query = [[[TTGeometrySearchQueryBuilder createWithTerm:term searchShapes:self.shapes] withLimit:30] build];
    [search searchWithQuery:query completionHandler:^(TTGeometrySearchResponse *response, TTResponseError *error) {
        if(!error) {
            [self displayResponse:response];
        }
    }];
    
    self.geometrySearch = search;
}

- (void)displayResponse:(TTGeometrySearchResponse *)response {
    for(TTSearchResult *result in response.results) {
        TTAnnotation *annotation = [TTAnnotation annotationWithCoordinate:result.position];
        [annotation setCanShowCallout:NO];
        [self.mapView.annotationManager addAnnotation:annotation];
    }
}

- (void)createShapesForSearch{
    self.shapes = [NSMutableArray array];

    // the outline of the Amsterdam district
    CLLocationCoordinate2D coordinates[13];
    coordinates[0] = CLLocationCoordinate2DMake(52.37874, 4.90482);
    coordinates[1] = CLLocationCoordinate2DMake(52.37664, 4.92559);
    coordinates[2] = CLLocationCoordinate2DMake(52.37497, 4.94877);
    coordinates[3] = CLLocationCoordinate2DMake(52.36805, 4.97246);
    coordinates[4] = CLLocationCoordinate2DMake(52.34918, 4.95993);
    coordinates[5] = CLLocationCoordinate2DMake(52.34016, 4.95169);
    coordinates[6] = CLLocationCoordinate2DMake(52.32894, 4.91392);
    coordinates[7] = CLLocationCoordinate2DMake(52.34048, 4.88611);
    coordinates[8] = CLLocationCoordinate2DMake(52.33953, 4.84388);
    coordinates[9] = CLLocationCoordinate2DMake(52.37067, 4.8432);
    coordinates[10] = CLLocationCoordinate2DMake(52.38492, 4.84663);
    coordinates[11] = CLLocationCoordinate2DMake(52.40011, 4.85058);
    coordinates[12] = CLLocationCoordinate2DMake(52.38995, 4.89075);
    self.mapPolygon = [TTPolygon polygonWithCoordinates:coordinates count:13 opacity:1 color:TTColorOverlay colorOutline:TTColorOverlayOutline];
    [self.mapView.annotationManager addOverlay:self.mapPolygon];
    TTSearchPolygon *searchPolygon = [TTSearchPolygon polygonWithCoordinates:coordinates count:13];
    [self.shapes addObject:searchPolygon];

    self.mapCircle = [TTCircle circleWithCenterCoordinate:COORDINATE_AMSTERDAM_CIRCLE radius:2000 width:2 color:TTColorOverlay fill:YES colorOutlet:TTColorOverlayOutline];
    [self.mapView.annotationManager addOverlay:self.mapCircle];
    TTSearchCircle *searchCircle = [TTSearchCircle circleWithCenter:COORDINATE_AMSTERDAM_CIRCLE radius:2000];
    [self.shapes addObject:searchCircle];
}

- (void)optionButtonTag:(NSInteger)tag index:(int)index {

    // Cleanup markers
    id<TTAnnotationManager> annotationManager = self.mapView.annotationManager;
    [annotationManager removeAllAnnotations];

    NSString *term = @"";
    switch (index) {
        case 0:
            term = @"Parking"; break;
        case 1:
            term = @"ATM"; break;
        case 2:
            term = @"Grocery"; break;
        default:
            break;
    }
    [self performSearchForTerm:term];
}

@end
