/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "ManeuverViewController.h"
#import <TomTomOnlineSDKRouting/TomTomOnlineSDKRouting.h>
#import "Locations.h"
#import "ProgressUtil.h"
#import "ManeuverIconGenerator.h"
#import "HeaderView.h"
#import "ManeuverCell.h"
#import "ETAWithSegmentView.h"
#define TableViewHeaderHeight 79.0f

static NSString *simpleTableIdentifier = @"ManeuverTableItem";
@interface ManeuverViewController () < ETAWithSegmentViewDelegate,TTRouteResponseDelegate>


@property (nonatomic, strong) ProgressUtil *progress;
@property (nonatomic, strong) TTRoute *routePlanner;
@property (nonatomic, strong) NSArray<TTInstruction *> *instructions;
@property (nonatomic, weak) HeaderView *headerView;
@property (nonatomic, strong) ETAWithSegmentView* ETAWithSegmentView;
@end

@implementation ManeuverViewController

-(void) setLanguage:(TTLanguage)language{
    [self planRoute:language];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [self buildBackButton];
    [self buildHeader:@"Maneuvers list" and:@"Amsterdam to Berlin"];
    
    self.instructions = [NSArray array];
    self.routePlanner = [[TTRoute alloc] init];
    self.routePlanner.delegate = self;
    
    [self.tableView registerClass:[ManeuverCell class] forCellReuseIdentifier:simpleTableIdentifier];
    self.tableView.rowHeight = 70;
    
    [self planRoute:TTLanguageEnglishGB];
    self.progress = [[ProgressUtil alloc] initWithViewController:self title:@"Routing..." subtitle:nil];

    self.ETAWithSegmentView = (ETAWithSegmentView *)[[[NSBundle mainBundle] loadNibNamed:@"ETAWithSegmentView" owner:self options:nil] firstObject];
    self.ETAWithSegmentView.delegate = self;
}

- (void)buildHeader:(NSString*)TitleB and:(NSString*) SubtitleB{
    HeaderView *headerView = [[HeaderView alloc] init];
    self.headerView = headerView;
    self.headerView.title.text = TitleB;
    self.headerView.subtitle.text = SubtitleB;
    self.navigationItem.titleView = self.headerView;
    [self updateHeaderViewLayout];
}

- (void)updateHeaderViewLayout {
    CGFloat _width = CGRectGetWidth(self.view.frame);
    CGFloat _height = 40.0;

    [self.navigationItem.titleView removeConstraints:[self.navigationItem.titleView constraints]];

    [self.navigationItem.titleView addConstraints:@[[NSLayoutConstraint constraintWithItem:self.navigationItem.titleView attribute:NSLayoutAttributeHeight relatedBy:NSLayoutRelationEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:_height], [NSLayoutConstraint constraintWithItem:self.navigationItem.titleView attribute:NSLayoutAttributeWidth relatedBy:NSLayoutRelationEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:_width]]];
    [self.navigationItem.titleView setFrame:CGRectMake(0, 0, _width, _height)];
    [self.navigationItem.titleView setNeedsUpdateConstraints];
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator {
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];
    [coordinator animateAlongsideTransition:^(id<UIViewControllerTransitionCoordinatorContext>UNUSED_PARAM _Nonnull context) {
        self.headerView.hidden = YES;
    } completion:^(id<UIViewControllerTransitionCoordinatorContext> UNUSED_PARAM _Nonnull context) {
        [self updateHeaderViewLayout];
        self.headerView.hidden = NO;
    }];
}

- (void)buildBackButton {
    UIImage *menuImage = [[UIImage imageNamed:@"BtnPixelDot"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal];
    UIBarButtonItem *optionsItem = [[UIBarButtonItem alloc] initWithImage:menuImage
                                                                    style:UIBarButtonItemStylePlain
                                                                   target:self.navigationController
                                                                   action:@selector(popViewControllerAnimated:)];
    self.navigationItem.leftBarButtonItem = optionsItem;
}

- (void)cancelProgress {
    [[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:NO];
    [self.progress cancelProgress];
}

- (void)displayProgressDialog {
    [[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:YES];
    [self.progress displayProgressWithAction:^{
        [self.routePlanner cancel];
        [self.navigationController popViewControllerAnimated:YES];
    }];
}

- (void)planRoute:(TTLanguage) language{
    TTRouteQuery *query = [[[[TTRouteQueryBuilder createWithDest:COORDINATE_BERLIN andOrig:COORDINATE_AMSTERDAM] withInstructionsType:TTOptionInstructionsTypeText] withLanguage:language]build];
    [_routePlanner planRouteWithQuery:query];
    [self displayProgressDialog];
}

- (void)route:(UNUSED_PARAM TTRoute *)route completedWithResult:(TTRouteResult *)result {
    [self cancelProgress];
    TTSummary *summary = result.routes.firstObject.summary;
    
    NSDate *eta = summary.arrivalTime;
    NSCalendar *calendar = [NSCalendar currentCalendar];
    [calendar setTimeZone: [NSTimeZone localTimeZone]];
    NSDateComponents *components = [calendar components:(NSCalendarUnitHour | NSCalendarUnitMinute) fromDate:eta];
    NSString *dateInString = [NSString stringWithFormat:@"%02ld:%02ld", components.hour, components.minute];
    
    [self.ETAWithSegmentView.etaView updateETA:dateInString kmToDistance:summary.lengthInMeters.integerValue / 1000];
    self.instructions = result.routes.firstObject.guidance.instructions;
    [self.tableView reloadData];
}
- (void)route:(UNUSED_PARAM TTRoute *)route completedWithResponseError:(UNUSED_PARAM TTResponseError *)resoultError {
    [self cancelProgress];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    ManeuverCell *cell = [tableView dequeueReusableCellWithIdentifier:simpleTableIdentifier];
    cell.maneuverImage.image = [ManeuverIconGenerator imageForInstruction:self.instructions[indexPath.row]];
    cell.maneuverDescription.text = self.instructions[indexPath.row].message;
    int distanceToManeuver = self.instructions[indexPath.row].routeOffsetInMeters.intValue;
    cell.maneuverDistance.text = distanceToManeuver > 1000 ? [NSString stringWithFormat:@"%d km", distanceToManeuver / 1000] : [NSString stringWithFormat:@"%d m", distanceToManeuver];
    return cell;
}

-(NSInteger)numberOfSectionsInTableView:(UNUSED_PARAM UITableView *)tableView {
    return 1;
}

-(NSInteger)tableView:(UNUSED_PARAM UITableView *)tableView numberOfRowsInSection:(UNUSED_PARAM NSInteger)section {
    return (NSInteger) self.instructions.count;
}

-(UIView *)tableView:(UNUSED_PARAM UITableView *)tableView viewForHeaderInSection:(UNUSED_PARAM NSInteger)section {
    return self.ETAWithSegmentView;
}

- (CGFloat)tableView:(UNUSED_PARAM UITableView *)tableView heightForHeaderInSection:(UNUSED_PARAM NSInteger)section {
    return TableViewHeaderHeight;
}

@end
