/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "MapLayersViewController.h"
#import <TomTomOnlineSDKMapsUIExtensions/TomTomOnlineSDKMapsUIExtensions.h>
#import "Locations.h"

@interface MapLayersViewController () <UIExampleViewControllerDelegate, TTMapViewDelegate>

@property (nonatomic, weak) IBOutlet TTMapView *mapView;
@property (weak, nonatomic) IBOutlet TTControlView *controlView;
@property (nonatomic, assign) MapLayersMode mode;
@property (nonatomic, assign) BOOL trafficOn;
@property (nonatomic, assign) BOOL incidentsOn;

@property (nonatomic, assign) TTMapTilesType mapTiles;

@end

@implementation MapLayersViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.delegate = self;
    self.mapView.showsUserLocation = YES;
    self.controlView.mapView = self.mapView;
    [self.controlView initDefaultCompassButton];
    [self.controlView initDefaultCenterButton];
    self.controlView.topLayoutConstraintCompassButton.constant = TopLayoutConstraintCompass;
    self.controlView.bottomLayoutConstraintCenterButton.constant = BottomLayoutConstraintCenterCustom;

    switch (_mode) {
        case MapLayersMapTiles:
            [self.mapView centerOnCoordinate:COORDINATE_AMSTERDAM withZoom:10];
            [self buildHeader:@"Map tiles" and:@"Amsterdam"];
            [self createSegmentedControllWithTag:MapLayersMapTiles options:@[@"Vector", @"Raster"] multiSelect:NO select:0];
            break;
        case MapLayersVectorMapAndTraffic:
            [self buildHeader:@"Vector map and traffic" and:@"London"];
            [self.mapView centerOnCoordinate:COORDINATE_LONDON withZoom:12];
            self.mapView.trafficTileStyle = [TTVectorTileType setStyle:TTVectorStyleRelative];
            [self createSegmentedControllWithTag:MapLayersVectorMapAndTraffic options:@[@"Flow", @"No traffic"] multiSelect:NO select:1];
            break;
        case MapLayersRasterMapAndTraffic:
            [self buildHeader:@"Raster map and traffic" and:@"London"];
            [self.mapView setTilesType:TTMapTilesRaster];
            [self.mapView centerOnCoordinate:COORDINATE_LONDON withZoom:12];
            self.mapView.trafficTileStyle = [TTRasterTileType setStyle:TTRasterStyleRelative];
            self.mapView.trafficIncidentsStyle = TTTrafficIncidentsStyleRaster;
            [self createSegmentedControllWithTag:MapLayersRasterMapAndTraffic options:@[@"Incidents", @"Flow", @"No traffic"] multiSelect:YES select:2];
            break;
        case MapLayersLanguageParameter:
            [self buildHeader:@"Map language" and:@"Europe"];
            [self.mapView centerOnCoordinate:COORDINATE_LODZ_ZEROMSKIEGO withZoom:3.2];
            [self createSegmentedControllWithTag:MapLayersLanguageParameter options:@[@"English", @"Russian", @"Dutch"] multiSelect:NO select:0];
            [self.mapView setMapLanguage:TTLanguageEnglishGB];
            break;
        case MapLayersGeopoliticalParameter:
            [self.mapView centerOnCoordinate: COORDINATE_ISRAEL withZoom:7];
            [self buildHeader:@"Geopolitical view" and:@"Disputed borders in Izrael"];
            [self createSegmentedControllWithTag:MapLayersGeopoliticalParameter options:@[@"Unified", @"Local"] multiSelect:NO select:0];
            break;
        case MapLayersCustomStyle:
            [self.mapView centerOnCoordinate:COORDINATE_AMSTERDAM withZoom:10];
            [self buildHeader:@"Custom styles" and:@"Amsterdam"];
            [self createSegmentedControllWithTag:MapLayersCustomStyle options:@[@"Basic", @"Custom"] multiSelect:NO select:0];
            break;
        default:
            break;
    }

}

- (void)setMapLayersMode:(MapLayersMode)mode {
    self.mode = mode;
}

#pragma mark - Result Map Tiles
- (void)resultMapTiles:(NSInteger)index {
    switch (index) {
        case 0:
            self.mapTiles = TTMapTilesVector;
            break;
        case 1:
            self.mapTiles = TTMapTilesRaster;
            break;
        default:
            break;
    }
    [self.mapView setTilesType:self.mapTiles];
}

#pragma mark - Result Vector TrafficLayers
- (void)resultVectorTrafficLayers:(NSInteger)index {
    switch (index) {
        case 0:
            self.mapView.trafficFlowOn = YES;
            break;
        case 1:
            self.mapView.trafficFlowOn = NO;
            break;
        default:
            break;
    }
}

- (void)resultRasterTrafficLayers:(NSInteger)index {
    [self.mapView setTilesType:TTMapTilesRaster];
    switch (index) {
        case 0:
            self.incidentsOn = !self.incidentsOn;
            self.mapView.trafficIncidentsOn = self.incidentsOn;
            break;
        case 1:
            self.trafficOn = !self.trafficOn;
            self.mapView.trafficFlowOn = self.trafficOn;
            break;
        case 2:
            self.trafficOn = NO;
            self.incidentsOn = NO;
            self.mapView.trafficFlowOn = self.trafficOn;
            self.mapView.trafficIncidentsOn = self.incidentsOn;
            break;
        default:
            break;
    }
    if (self.trafficOn == NO && self.incidentsOn == NO){
        self.option1.selected = true;
    }
}

- (void)resultLanguageParameter:(NSInteger)index {
    
    switch (index) {
        case 0:
            [self.mapView setMapLanguage:TTLanguageEnglishGB];
            break;
        case 1:
            [self.mapView setMapLanguage:TTLanguageRussian];
            break;
        case 2:
            [self.mapView setMapLanguage:TTLanguageDutchNL];
            break;
        default:
            break;
    }
}

- (void)resultGeopoliticalParameter:(NSInteger)index {

    switch (index) {
        case 0:
            [self.mapView centerOnCoordinate: COORDINATE_ISRAEL withZoom:7];
            [self.mapView setGeopoliticalView:TTGeoViewNone];
            break;
        case 1:
            
            [self.mapView setGeopoliticalView:TTGeoViewIL];
            break;
    }
}

#pragma mark - Result Custom Styles
- (void)resultMapCustomStyle:(NSInteger)index {
    switch (index) {
        case 0:
        [self.mapView setStylePath:nil];
        break;
        case 1:
        {
        NSString* customStyle = [[NSBundle mainBundle] pathForResource:@"style" ofType:@"json"];
        [self.mapView setStylePath:customStyle];
        }
        break;
        default:
        break;
    }
}

- (void)optionButtonTag:(NSInteger)tag index:(int)index {

    if (index == 0 || index == 1) {
        [self deselectOptionIndexs:@[@(0)]];
    } else {
        [self deselectOptionIndexs:@[@(2), @(1)]];
    }
    
    switch (tag) {
        case MapLayersMapTiles:
            self.mode = MapLayersMapTiles;
            [self resultMapTiles:index];
            break;

        case MapLayersVectorMapAndTraffic:
            self.mode = MapLayersVectorMapAndTraffic;
            [self resultVectorTrafficLayers:index];
            break;
        case MapLayersRasterMapAndTraffic:
            self.mode = MapLayersRasterMapAndTraffic;
            [self resultRasterTrafficLayers:index];
            break;
        case MapLayersLanguageParameter:
            self.mode = MapLayersLanguageParameter;
            [self resultLanguageParameter:index];
            break;
        case MapLayersGeopoliticalParameter:
            self.mode = MapLayersGeopoliticalParameter;
            [self resultGeopoliticalParameter:index];
            break;
        case MapLayersCustomStyle:
            self.mode = MapLayersCustomStyle;
            [self resultMapCustomStyle:index];
            break;
        default:
            break;
    }
    
}

@end
