/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "MapManipulationViewController.h"
#import <TomTomOnlineSDKMapsUIExtensions/TomTomOnlineSDKMapsUIExtensions.h>
#import "UIView+Toast.h"
#import "Locations.h"
#import "LocationManager.h"

@interface MapManipulationViewController() <UIExampleViewControllerDelegate, TTMapViewDelegate>

@property (nonatomic, weak) IBOutlet TTMapView *mapView;
@property (weak, nonatomic) IBOutlet TTControlView *controlView;
@property (nonatomic, assign) MapManipulationMode mode;

@property (nonatomic, assign) BOOL perspective3D;
@property (nonatomic, assign) CLLocationCoordinate2D currentLocation;

@end

@implementation MapManipulationViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self.mapView centerOnCoordinate:COORDINATE_AMSTERDAM withZoom:10];
    self.mapView.delegate = self;
    self.mapView.showsUserLocation = YES;
    self.controlView.mapView = self.mapView;
    [self.controlView initDefaultCompassButton];
    [self.controlView initDefaultCenterButton];
    self.controlView.topLayoutConstraintCompassButton.constant = TopLayoutConstraintCompass;
    self.controlView.bottomLayoutConstraintCenterButton.constant = BottomLayoutConstraintCenterCustom;
    self.delegate = self;

    switch (_mode) {
        case MapManipulationMap2D3D:
            [self buildHeader:@"Map perspective" and:@"Amsterdam"];
            [self createSegmentedControllWithTag:MapManipulationMap2D3D options:@[@"2D mode", @"3D mode"] multiSelect:NO select:0];
            break;

        case MapManipulationMapCenter:
            [self buildHeader:@"Map centering" and:@"Amsterdam"];
            [self createSegmentedControllWithTag:MapManipulationMapCenter options:@[@"Amsterdam", @"Berlin", @"London"] multiSelect:NO select:0];
            break;

        case MapManipulationMapEvents:
            [self buildHeader:@"Map events" and:@"Amsterdam"];
            self.showToastEnable = YES;
            self.controlView.bottomLayoutConstraintCenterButton.constant = -25;
            break;
        default:
            break;
    }
}



- (void)setMapManipulationMode:(MapManipulationMode)mode {
    self.mode = mode;
}

#pragma mark - Result Map 2D / 3D
- (void)resultMap2D3D:(NSInteger)index {
    self.perspective3D = false;
    self.mode = MapManipulationMap2D3D;
    switch (index) {
        case 0:
            self.perspective3D = false;
            break;

        case 1:
            self.perspective3D = true;
            break;

        default:
            break;
    }

    [self.mapView setPerspective3D: self.perspective3D];

}

#pragma mark - Result Centering The Map
- (void)resultCenterMap:(NSInteger)index {
    self.currentLocation = [[[LocationManager sharedInstance] location] coordinate];
    self.mode = MapManipulationMapCenter;
    switch (index) {
        case 0: // Amsterdam
            self.currentLocation = COORDINATE_AMSTERDAM;
            [self setSubtitle:@"Amsterdam"];
            break;
        case 1: // Berlin
            self.currentLocation = COORDINATE_BERLIN;
            [self setSubtitle:@"Berlin"];
            break;
        case 2: // GPS
            self.currentLocation = COORDINATE_LONDON;
            [self setSubtitle:@"London"];
            break;
        default:
            break;
    }

    [self.mapView centerOnCoordinate:self.currentLocation withZoom:10];
}

- (void)optionButtonTag:(NSInteger)tag index:(int)index {
    switch (tag) {
        case MapManipulationMap2D3D:
            [self resultMap2D3D:index];
            break;

        case MapManipulationMapCenter:
            [self resultCenterMap:index];
            break;
        default:
            break;
    }
}

- (void)mapView:(nonnull TTMapView *)mapView didDoubleTap:(CLLocationCoordinate2D)coordinate {
    [self showToast:[NSString stringWithFormat:@"Double Press: %.5f, %.5f", coordinate.latitude, coordinate.longitude] withTimer:YES];
}

- (void)mapView:(nonnull TTMapView *) UNUSED_PARAM mapView didSingleTap:(CLLocationCoordinate2D)coordinate {
    [self showToast:[NSString stringWithFormat:@"Single Press: %.5f, %.5f", coordinate.latitude, coordinate.longitude] withTimer:YES];
}

- (void)mapView:(nonnull TTMapView *)mapView didLongPress:(CLLocationCoordinate2D)coordinate {
    [self showToast:[NSString stringWithFormat:@"Long Press: %.5f, %.5f", coordinate.latitude, coordinate.longitude] withTimer:YES];
}

- (void)mapView:(TTMapView *)mapView didPanning:(CLLocationCoordinate2D)coordinate {
    [self showToast:[NSString stringWithFormat:@"Panning: %.5f, %.5f", coordinate.latitude, coordinate.longitude] withTimer:YES];
}

@end
