/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "OptionsViewController.h"
#import "UISearchViewController.h"
#import "MapViewController.h"
#import "MapManipulationViewController.h"
#import "GeometrySearchViewController.h"
#import "ReverseGeocodingViewController.h"
#import "MapAnnotationsViewController.h"
#import "MapLayersViewController.h"
#import "RouteViewController.h"
#import "UIMapUIExtensions.h"
#import "ManeuverViewController.h"
#import "AboutViewController.h"
#import "SearchAlongTheRouteViewController.h"
#import "EntryPointsViewController.h"
#import "AdditionalDataViewController.h"
#import "BatchSearchViewController.h"
#import "StaticImageViewController.h"
#import "TrafficViewController.h"
#define kCellIdentifier @"CellIdentifier"

@interface OptionsViewController ()

@property (nonatomic) IBOutlet UITableView *tableView;
@property (nonatomic) NSArray *dataSource;

@end

@implementation OptionsViewController

+ (NSString *)versionApp {
    NSString *appName = [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleName"];
    NSString *number = [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleShortVersionString"];
    NSString *version =[[NSBundle mainBundle] objectForInfoDictionaryKey:(__bridge NSString*) kCFBundleVersionKey];
    return [NSString stringWithFormat:@"%@ %@ (%@)", appName, number, version];
}

+ (void)initialize {
    menuSectionDescriptions = @{@(MenuSectionAbout): @"",
                                @(MenuSectionSearch): @"Search",
                                @(MenuSectionMapManipulation): @"Map manipulation",
                                @(MenuSectionAnnotation): @"Map annotations",
                                @(MenuSectionTraffic): @"Traffic",
                                @(MenuSectionRoute): @"Routing",
                                @(MenuSectionMapLayers): @"Map display",
                                @(MenuSectionVersion): [OptionsViewController versionApp],
                                };

    menuSectionSearchDescriptions = @{@(MenuItemSectionSearchAddress): @"Address search",
                                      @(MenuItemSectionSearchCategory): @"Category search",
                                      @(MenuItemSectionSearchAutocomplete): @"Typeahead parameter",
                                      @(MenuItemSectionSearchLanguage): @"Language selector",
                                      @(MenuItemSectionSearchFuzzy) : @"Max fuzziness levels",
                                      @(MenuItemSectionSearchReverseGeocoding): @"Reverse geocoding",
                                      @(MenuItemSectionSearchAlongTheRoute):@"Search along the route",
                                      @(MenuItemSectionSearchGeometry): @"Geometry search",
                                      @(MenuItemSectionSearchEntryPoints): @"Entry points",
                                      @(MenuItemSectionSearchAdditionalData): @"Additional Data",
                                      @(MenuItemSectionSearchBatch): @"Batch search"
                                      };

    menuSectionMapManipluationDescriptions = @{@(MenuItemSectionMapManipulationMapEvents): @"Map events",
                                               @(MenuItemSectionMapManipulationMapViewPerspective): @"Map perspective",
                                               @(MenuItemSectionMapManipulationCenteringTheMap): @"Map centering",
                                               @(MenuItemSectionMapManipulationUIExtension): @"Map UI extensions"
                                               };

    menuSectionAnnotations = @{@(MenuItemSectionAnnotationMarkers): @"Markers",
                               @(MenuItemSectionAnnotationAdvancedMarkers): @"Advanced markers",
                               @(MenuItemSectionAnnotationShapes): @"Shapes",
                               @(MenuItemSectionAnnotationBalloons): @"Balloons",
                               @(MenuItemSectionAnnotationMarkersClustering): @"Markers clustering"
                               };

    menuSectionMapLayersDescriptions = @{@(MenuItemSectionMapLayersMapTilesLayer): @"Map tiles",
                                         @(MenuItemSectionMapLayersVectorMapAndTraffic): @"Vector map and traffic",
                                         @(MenuItemSectionMapLayersRasterMapAndTraffic): @"Raster map and traffic",
                                         @(MenuItemSectionMapLayersLanguageParameter): @"Map language",
                                         @(MenuItemSectionMapLayersGeopoliticalParameter): @"Geopolitical view",
                                         @(MenuItemSectionMapLayersCustomStyle): @"Custom styles",
                                         @(MenuItemSectionMapLayersStaticImage): @"Static image"
                                         };

    menuSectionRouteDescriptions = @{@(MenuItemSectionRouteTravelModes): @"Travel modes",
                                     @(MenuItemSectionRouteRouteTypes): @"Route types",
                                     @(MenuItemSectionRouteRouteAvoids): @"Route avoids",
                                     @(MenuItemSectionRouteRouteWithWaypoints): @"Route with waypoints",
                                     @(MenuItemSectionRouteManeuverList): @"Maneuver list",
                                     @(MenuItemSectionRouteTimeTravel): @"Departure/arrival time",
                                     @(MenuItemSectionRouteAlternativeRoutes): @"Alternative routes",
                                     @(MenuItemSectionRouteConsumptionModel): @"Consumption model",
                                     @(MenuItemSectionRouteSupportingPoints): @"Supporting points",
                                     @(MenuItemSectionRouteReachableRange): @"Reachable range"
                                     };

    menuSectionTrafficDescriptions = @{
                                       @(MenuItemSectionTrafficIncidentsList): @"Traffic incidents list",
                                       };


    menuSectionOtherDescriptions = @{@(MenuItemSectionOtherAbout): @"About"};
}

- (void)viewDidLoad {
    [super viewDidLoad];

    self.tableView.backgroundView = nil;
    self.tableView.backgroundColor = [UIColor whiteColor];

    self.title = @"Functional Examples";
    self.navigationController.interactivePopGestureRecognizer.enabled = YES;

    UIView *headerView = [[NSBundle mainBundle] loadNibNamed:@"OptionsTableHeader" owner:self options:nil][0];
    headerView.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    self.tableView.tableHeaderView = headerView;

    [self.navigationItem setHidesBackButton:YES animated:YES];
}

- (BOOL)prefersStatusBarHidden {
    return NO;
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [[self navigationController] setNavigationBarHidden:NO animated:YES];
    self.navigationController.hidesBarsOnTap = NO;
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
}


#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UNUSED_PARAM UITableView *)tableView {
    return MenuSectionTotal;
}

- (NSInteger)tableView:(UNUSED_PARAM UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    switch (section) {

        case MenuSectionSearch:
            return MenuItemSectionSearchTotal;

        case MenuSectionMapLayers:
            return MenuItemSectionMapLayersTotal;

        case MenuSectionMapManipulation:
            return MenuItemSectionMapManipluationTotal;

        case MenuSectionAnnotation:
            return MenuItemSectionAnnotationTotal;

        case MenuSectionTraffic:
            return MenuItemSectionTrafficTotal;

        case MenuSectionRoute:
            return MenuItemSectionRouteTotal;

        case MenuSectionAbout:
            return MenuItemSectionOtherTotal;

        default:
            return 0;
    }
}

- (NSString *)tableView:(UNUSED_PARAM UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    return [menuSectionDescriptions objectForKey:@(section)];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:kCellIdentifier];

    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:kCellIdentifier];
    }

    cell.textLabel.text = [self textForCell:indexPath];
    cell.textLabel.accessibilityIdentifier = cell.textLabel.text;

    return cell;
}

- (NSString *)textForCell:(NSIndexPath *)indexPath {
    switch (indexPath.section) {

        case MenuSectionSearch:
            return [menuSectionSearchDescriptions objectForKey:@(indexPath.row)];
        case MenuSectionMapLayers:
            return [menuSectionMapLayersDescriptions objectForKey:@(indexPath.row)];
        case MenuSectionMapManipulation:
            return [menuSectionMapManipluationDescriptions objectForKey:@(indexPath.row)];
        case MenuSectionRoute:
            return [menuSectionRouteDescriptions objectForKey:@(indexPath.row)];
        case MenuSectionAnnotation:
            return [menuSectionAnnotations objectForKey:@(indexPath.row)];
        case MenuSectionTraffic:
            return [menuSectionTrafficDescriptions objectForKey:@(indexPath.row)];
        case MenuSectionAbout:
            return [menuSectionOtherDescriptions objectForKey:@(indexPath.row)];

        default:
            return nil;
    }
}

#pragma mark - Table view delegate
- (void)tableView:(UNUSED_PARAM UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [self.tableView deselectRowAtIndexPath:indexPath animated:YES];

    if(indexPath.section == MenuSectionSearch) {
        if (indexPath.row == MenuItemSectionSearchReverseGeocoding) {
            [self showViewController:[self reverseGeocoding] sender:self];
        } else if (indexPath.row == MenuItemSectionSearchGeometry) {
            [self showViewController:[self polygonSearch] sender:self];
        } else if (indexPath.row == MenuItemSectionSearchAlongTheRoute) {
            [self showViewController:[self searchAlongTheRoute] sender:self];
        } else if (indexPath.row == MenuItemSectionSearchEntryPoints) {
            [self showViewController:[self entryPoints] sender:self];
        } else if (indexPath.row == MenuItemSectionSearchAdditionalData) {
            [self showViewController:[self additionalData] sender:self];
        } else if (indexPath.row == MenuItemSectionSearchBatch) {
            [self showViewController:[self batchSearch] sender:self];
        } else {
            [self showViewController:[self searchViewControllerWithMode:(SearchMode)indexPath.row] sender:self];
        }
    } else if(indexPath.section == MenuSectionAnnotation) {
        [self showViewController:[self mapMapAnnotationsWithMode:(MapAnnotationMode)indexPath.row] sender:self];
    } else if (indexPath.section == MenuSectionMapLayers) {
        if (indexPath.row == MenuItemSectionMapLayersStaticImage) {
            [self showViewController:[self staticImageViewController] sender:self];
        } else {
            [self showViewController:[self mapLayersWithMode:(MapLayersMode)indexPath.row] sender:self];
        }
    } else if (indexPath.section == MenuSectionMapManipulation) {
        if (indexPath.row == MenuItemSectionMapManipulationUIExtension) {
            [self showViewController:[self compassAndCurrentLocationViewController] sender:nil];
        } else {
            [self showViewController:[self mapManipulationWithMode:(MapManipulationMode)indexPath.row] sender:self];
        }
    } else if (indexPath.section == MenuSectionRoute){
        if(indexPath.row == MenuItemSectionRouteManeuverList) {
            [self showViewController:[self maneuverViewController] sender:self];
        } else {
            [self showViewController:[self routeWithMode:(RouteMode)indexPath.row] sender:self];
        }
    } else if (indexPath.section == MenuSectionAbout){
        [self showViewController:[self aboutViewController] sender:self];
    } else if (indexPath.section == MenuSectionTraffic) {
        [self showViewController:[self trafficViewController] sender:self];
    } else {
        MapViewController *mapVC = [[MapViewController alloc] init];
        [self showViewController:mapVC sender:self];
    }
}

#pragma mark - UI

- (TrafficViewController *)trafficViewController {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    TrafficViewController *tvc = (TrafficViewController *)[sb instantiateViewControllerWithIdentifier:@"TrafficViewController"];
    return tvc;
}

- (StaticImageViewController *)staticImageViewController {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    StaticImageViewController *sivc = (StaticImageViewController *)[sb instantiateViewControllerWithIdentifier:@"StaticImageViewController"];
    return sivc;
}

- (UISearchViewController *)searchViewControllerWithMode:(SearchMode)index {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    UISearchViewController *svc = (UISearchViewController *)[sb instantiateViewControllerWithIdentifier:@"UISearchViewController"];
    [svc setSearchMode:(SearchMode)index];
    return svc;
}

- (MapAnnotationsViewController *)mapMapAnnotationsWithMode:(MapAnnotationMode)index  {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    MapAnnotationsViewController *mavc = (MapAnnotationsViewController *)[sb instantiateViewControllerWithIdentifier:@"MapAnnotationsViewController"];
    [mavc setMapAnnotationsMode:(MapAnnotationMode)index];
    return mavc;
}

- (MapManipulationViewController *)mapManipulationWithMode:(MapManipulationMode)index {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    MapManipulationViewController *mmvc = (MapManipulationViewController *)[sb instantiateViewControllerWithIdentifier:@"MapManipulationViewController"];
    [mmvc setMapManipulationMode:(MapManipulationMode)index];
    return mmvc;
}

- (UIMapUIExtensions *)compassAndCurrentLocationViewController {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    UIMapUIExtensions *caclvc = (UIMapUIExtensions *)[sb instantiateViewControllerWithIdentifier:@"UIMapUIExtensions"];
    return caclvc;
}

- (MapLayersViewController *)mapLayersWithMode:(MapLayersMode)index {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    MapLayersViewController *mlvc = (MapLayersViewController *)[sb instantiateViewControllerWithIdentifier:@"MapLayersViewController"];
    [mlvc setMapLayersMode:(MapLayersMode)index];
    return mlvc;
}

- (GeometrySearchViewController *)polygonSearch {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    GeometrySearchViewController *psvc = (GeometrySearchViewController *)[sb instantiateViewControllerWithIdentifier:@"GeometrySearchViewController"];
    return psvc;
}

- (AdditionalDataViewController *)additionalData {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    AdditionalDataViewController *advc = (AdditionalDataViewController *)[sb instantiateViewControllerWithIdentifier:@"AdditionalDataViewController"];
    return advc;
}

- (BatchSearchViewController *)batchSearch {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    BatchSearchViewController *bsvc = (BatchSearchViewController *)[sb instantiateViewControllerWithIdentifier:@"BatchSearchViewController"];
    return bsvc;
}

- (EntryPointsViewController *)entryPoints {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    EntryPointsViewController *epvc = (EntryPointsViewController *)[sb instantiateViewControllerWithIdentifier:@"EntryPointsViewController"];
    return epvc;
}

- (ReverseGeocodingViewController *)reverseGeocoding {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    ReverseGeocodingViewController *rgvc = (ReverseGeocodingViewController *)[sb instantiateViewControllerWithIdentifier:@"ReverseGeocodingViewController"];
    return rgvc;
}

- (SearchAlongTheRouteViewController *)searchAlongTheRoute {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    SearchAlongTheRouteViewController *satr = (SearchAlongTheRouteViewController *)[sb instantiateViewControllerWithIdentifier:@"SearchAlongTheRouteViewController"];
    return satr;
}

- (RouteViewController *)routeWithMode:(RouteMode)index {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    RouteViewController *rvc = (RouteViewController *)[sb instantiateViewControllerWithIdentifier:@"RouteViewController"];
    [rvc setRouteMode: index];
    return rvc;
}

- (ManeuverViewController *)maneuverViewController {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    ManeuverViewController *mvc = (ManeuverViewController *)[sb instantiateViewControllerWithIdentifier:@"ManeuverViewController"];
    return mvc;
}

- (AboutViewController *)aboutViewController {
    UIStoryboard *sb = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    AboutViewController *avc = (AboutViewController *)[sb instantiateViewControllerWithIdentifier:@"AboutViewController"];
    return avc;
}

@end
