/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "ReverseGeocodingViewController.h"
#import <TomTomOnlineSDKMapsUIExtensions/TomTomOnlineSDKMapsUIExtensions.h>
#import <TomTomOnlineSDKSearch/TomTomOnlineSDKSearch.h>
#import <TomTomOnlineUtils/TomTomOnlineUtils.h>

@interface ReverseGeocodingViewController () <TTMapViewDelegate, TTAnnotationDelegate>

@property (nonatomic, weak) IBOutlet TTMapView *mapView;
@property (weak, nonatomic) IBOutlet TTControlView *controlView;
@property (nonatomic, strong) TTReverseGeocoder *reverseGeocoder;
@property (nonatomic, strong) NSString *result;

@end

@implementation ReverseGeocodingViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self buildHeader:@"Reverse geocoding" and:@""];
    self.mapView.delegate = self;
    self.mapView.showsUserLocation = YES;
    self.controlView.mapView = self.mapView;
    [self.controlView initDefaultCompassButton];
    [self.controlView initDefaultCenterButton];
    self.controlView.topLayoutConstraintCompassButton.constant = TopLayoutConstraintCompass;
    self.controlView.bottomLayoutConstraintCenterButton.constant = BottomLayoutConstraintCenterCustom;
    self.reverseGeocoder = [[TTReverseGeocoder alloc] init];
    [self.mapView.annotationManager setDelegate:self];
}

- (void)mapView:(UNUSED_PARAM TTMapView * _Nonnull)mapView didLongPress:(CLLocationCoordinate2D)coordinate {
    TTReverseGeocoderQuery *query =
    [[[[TTReverseGeocoderQueryBuilder createWithCLLocationCoordinate2D:coordinate]
        withReturnRoadUse:true]
      withReturnSpeedLimit:true] build];

    [self.mapView.annotationManager removeAllAnnotations];
    self.result = NSLocalizedString(@"Loading ...", "");
    TTAnnotation *annotation = [TTAnnotation annotationWithCoordinate:coordinate];
    [self.mapView.annotationManager addAnnotation:annotation];

    [self.reverseGeocoder reverseGeocoderWithQuery:query completionHandle:^(TTReverseGeocoderResponse * _Nullable response, TTResponseError * _Nullable error) {
        TTReverseGeocoderFullAddress *fullAddress = response.result.addresses.firstObject;
        if (fullAddress) {
            self.result = [self addressString: fullAddress.address];
            [self.mapView.annotationManager selectAnnotation:annotation];
        }
    }];
}

- (void)mapView:(TTMapView *)UNUSED_PARAM mapView didPanning:(CLLocationCoordinate2D) UNUSED_PARAM coordinate {}
- (void)mapView:(TTMapView *)UNUSED_PARAM mapView didDoubleTap:(CLLocationCoordinate2D) UNUSED_PARAM coordinate {}
- (void)mapView:(TTMapView *)UNUSED_PARAM mapView didSingleTap:(CLLocationCoordinate2D)UNUSED_PARAM coordinate {}

- (UIView<TTCalloutView> *)annotationManager:(id<TTAnnotationManager>)annotationManager viewForSelectedAnnotation:(TTAnnotation *)selectedAnnotation {
    return [[TTCalloutViewSimple alloc] initWithText:self.result];
}

- (NSString *)addressString:(TTAddress *)address {
    NSMutableString *description = [NSMutableString new];
    [self appendString:description textString:address.buildingNumber];
    [self appendString:description textString:address.streetNameAndNumber];
    [self appendString:description textString:address.speedLimit];
    [self appendString:description textString:address.country];
    [self appendString:description textString:address.countryCode];
    [self appendString:description textString:address.countryCodeISO3];
    [self appendString:description textString:address.countrySubdivision];
    [self appendString:description textString:address.countrySubdivisionName];
    [self appendString:description textString:address.municipality];
    [self appendString:description textString:address.municipalitySubdivision];
    [self appendString:description textString:address.postalCode];
    [self appendString:description textString:address.freeformAddress];
    return description;
}

- (void)appendString:(NSMutableString*)sourceText textString:(NSString *)text {
    if (sourceText) {
        if ([sourceText length] > 0 && [text length] > 0) {
            [sourceText appendString:@", "];
        }
        if ([text length] > 0) {
            [sourceText appendString:text];
        }
    }
}

@end
