/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "StaticImageViewController.h"
#import <TomTomOnlineSDKMapsStaticImage/TomTomOnlineSDKMapsStaticImage.h>
#import "Locations.h"

@interface StaticImageViewController () <UICollectionViewDelegate, UICollectionViewDataSource, UIExampleViewControllerDelegate, UICollectionViewDelegateFlowLayout>

@property (nonatomic, weak) UICollectionView *staticImagesCollectionView;

@end

@implementation StaticImageViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self buildHeader:@"Static map" and:@"press map to check configuration"];
    self.delegate = self;
    [self setShowToastEnable:YES];
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return 6;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath {
    CGFloat frame = CGRectGetWidth(self.view.frame) < CGRectGetHeight(self.view.frame) ? CGRectGetWidth(self.view.frame) : CGRectGetHeight(self.view.frame);
    CGFloat width = frame * 0.495;
    return CGSizeMake(width, width);
}

- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    UICollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"staticImageCollectionCellView" forIndexPath:indexPath];
    UIImageView *view = [UIImageView new];
    [cell setBackgroundView:view];
    [cell setBackgroundColor:UIColor.whiteColor];
    switch (indexPath.row) {
        case 0:
            [self queryForAmsterdamCustomUIImageView:view];
            break;
        case 1:
            [self queryForAmsterdamStyleNightWithUIImageView:view];
            break;
        case 2:
            [self queryForAmsterdamZoom15WithUIImageView:view];
            break;
        case 3:
            [self queryForAmsterdamLayerHybridWithUIImageView:view];
            break;
        case 4:
            [self queryForAmsterdamStyleAzureWithUIImageView:view];
            break;
        case 5:
            [self queryForAmsterdamStyleNightZoom8WithUIImageView:view];
            break;
        default:
            break;

    }
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    switch (indexPath.row) {
        case 0:
            [self showToast:[NSString stringWithFormat:@"Default static map Amsterdam"] withTimer:YES];
            break;
        case 1:
            [self showToast:[NSString stringWithFormat:@"Night style Amsterdam"] withTimer:YES];
            break;
        case 2:
            [self showToast:[NSString stringWithFormat:@"Zoom 15 Amsterdam"] withTimer:YES];
            break;
        case 3:
            [self showToast:[NSString stringWithFormat:@"Hybrid map Amsterdam"] withTimer:YES];
            break;
        case 4:
            [self showToast:[NSString stringWithFormat:@"Custom style Amsterdam"] withTimer:YES];
            break;
        case 5:
            [self showToast:[NSString stringWithFormat:@"Zoom 8 night mode Amsterdam"] withTimer:YES];
            break;
        default:
            break;

    }
}

- (void)queryForAmsterdamCustomUIImageView:(UIImageView *)imageView {
    TTStaticImageQuery *siq = [[[[[[[TTStaticImageQueryBuilder withCenter:COORDINATE_AMSTERDAM]
                               withLayer:TTLayerTypeBasic]
                               withStyle:TTStyleTypeMain]
                                  withExt:TTExtTypePNG]
                               withHeight:512]
                               withWidth:512]
                               build];
    [self perfomQuery:siq withImageView:imageView];
}

- (void)queryForAmsterdamStyleNightWithUIImageView:(UIImageView *)imageView {
    TTStaticImageQuery *siq = [[[TTStaticImageQueryBuilder withCenter:COORDINATE_AMSTERDAM]
                                withStyle:TTStyleTypeNight]
                               build];
    [self perfomQuery:siq withImageView:imageView];
}

- (void)queryForAmsterdamZoom15WithUIImageView:(UIImageView *)imageView {
    TTStaticImageQuery *siq = [[[TTStaticImageQueryBuilder withCenter:COORDINATE_AMSTERDAM]
                                withZoomLevel:15]
                               build];
    [self perfomQuery:siq withImageView:imageView];
}

- (void)queryForAmsterdamLayerHybridWithUIImageView:(UIImageView *)imageView {
    TTStaticImageQuery *siq = [[[TTStaticImageQueryBuilder withCenter:COORDINATE_AMSTERDAM]
                                withLayer:TTLayerTypeHybrid]
                               build];
    [self perfomQuery:siq withImageView:imageView];
}

- (void)queryForAmsterdamStyleAzureWithUIImageView:(UIImageView *)imageView {
    TTStaticImageQuery *siq = [[[TTStaticImageQueryBuilder withCenter:COORDINATE_AMSTERDAM]
                                withCustomStyle:@"main-azure"]
                               build];
    [self perfomQuery:siq withImageView:imageView];
}

- (void)queryForAmsterdamStyleNightZoom8WithUIImageView:(UIImageView *)imageView {
    TTStaticImageQuery *siq = [[[[TTStaticImageQueryBuilder withCenter:COORDINATE_AMSTERDAM]
                                withStyle:TTStyleTypeNight]
                                withZoomLevel:8]
                               build];
    [self perfomQuery:siq withImageView:imageView];
}

- (void)perfomQuery:(TTStaticImageQuery *)query withImageView:(UIImageView *)imageView {
    [[TTStaticImage new] imageForQuery:query completionHandler:^(UIImage * _Nullable image, TTResponseError * _Nullable error) {
        [imageView setImage:image];
    }];
}

@end
