/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "UIMapUIExtensions.h"
#import <TomTomOnlineSDKMapsUIExtensions/TomTomOnlineSDKMapsUIExtensions.h>
#import "Locations.h"
#import "OverlayDrawing.h"
#import "MyCalloutView.h"

@interface UIMapUIExtensions() <UIExampleViewControllerDelegate>

@property (nonatomic, weak) IBOutlet TTMapView *mapView;
@property (nonatomic, weak) IBOutlet TTControlView *controlView;
@property (nonatomic, strong) UIButton *customCompassButton;
@property (nonatomic, strong) UIButton *customCenterButton;

@end

@implementation UIMapUIExtensions

- (void)viewDidLoad {
    [super viewDidLoad];
    self.delegate = self;
    self.mapView.showsUserLocation = YES;
    [self buildHeader:@"Map UI extensions" and:@"Amsterdam"];
    [self createSegmentedControllWithTag:0 options:@[@"Default", @"Custom", @"Hide"] multiSelect:NO select:0];
    TTCameraPosition *cameraPosition = [[TTCameraPosition alloc] initWithCamerPosition:COORDINATE_AMSTERDAM withAnimationDuration:0 withBearing:45 withPitch:0 withZoom:10];
    [self.mapView setCameraPosition:cameraPosition];
    [self setupControlView];
    
}

- (void)setupControlView {
    // The instance 'controlView' is a type of TTControllView
    self.controlView.mapView = self.mapView;
    [self defaultCompassAndCurrentLocationButtons];
}

- (void)optionButtonTag:(UNUSED_PARAM NSInteger)tag index:(int)index {
    if(index < 0) {
        return;
    }
    switch (index) {
        case 0:
            [self defaultCompassAndCurrentLocationButtons];
            break;
        case 1:
            [self customCompassAndCurrentLocationButtons];
            break;
        case 2:
            [self hideCompassAndCurrentLocationButtons];
            break;
        default:
            break;
    }
}

- (void)defaultCompassAndCurrentLocationButtons {
    [self.controlView initDefaultCompassButton];
    [self.controlView initDefaultCenterButton];
    [self.controlView initDefaultTTPanControlView];
    [self.controlView initDefaultTTZoomView];
    self.controlView.compassButton.hidden = NO;
    self.controlView.centerButton.hidden = NO;
    self.controlView.zoomView.hidden = NO;
    self.controlView.controlView.hidden = NO;
    
    
    self.controlView.topLayoutConstraintCompassButton.constant = TopLayoutConstraintCompass;
    self.controlView.bottomLayoutConstraintCenterButton.constant = BottomLayoutConstraintCenterCustom;
}

- (void)hideCompassAndCurrentLocationButtons {
    self.controlView.compassButton.hidden = YES;
    self.controlView.centerButton.hidden = YES;
    self.controlView.zoomView.hidden = YES;
    self.controlView.controlView.hidden = YES;
}

- (void)customCompassAndCurrentLocationButtons {
    
    self.customCenterButton = [self createCustomCenterButton];
    self.controlView.centerButton = self.customCenterButton;
    self.customCompassButton = [self createCustomComapssButton];
    self.controlView.compassButton = self.customCompassButton;
    [self creatrCustomZoomControll];
    [self creatrCustomPanControll];
    self.controlView.topLayoutConstraintCompassButton.constant = TopLayoutConstraintCompass;
    self.controlView.bottomLayoutConstraintCenterButton.constant = BottomLayoutConstraintCenterCustom;
}

- (UIButton *)createCustomCenterButton {
    UIButton *customCenterButton = [[UIButton alloc] initWithFrame:CGRectZero];
    [self.mapView addSubview:customCenterButton];
    [self setUpLayoutConstraintCenterButton:customCenterButton];
    UIImage *centerDefaultImage = [UIImage imageNamed:@"custom_current_location" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    UIImage *centerPressedImage = [UIImage imageNamed:@"custom_current_location" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    [customCenterButton setBackgroundImage:centerDefaultImage forState:UIControlStateNormal];
    [customCenterButton setBackgroundImage:centerPressedImage forState:UIControlStateHighlighted];
    [customCenterButton.layer setCornerRadius:customCenterButton.frame.size.width/2];
    [customCenterButton.layer setMasksToBounds:YES];
    return customCenterButton;
}

- (void)creatrCustomPanControll{

    UIButton *customLeftBtn = [[UIButton alloc] initWithFrame:CGRectZero];
    UIImage *leftNormal = [UIImage imageNamed:@"arrow_left_custom" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    UIImage *leftHighlighted = [UIImage imageNamed:@"arrow_left_custom" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    [customLeftBtn setBackgroundImage:leftNormal forState:UIControlStateNormal];
    [customLeftBtn setBackgroundImage:leftHighlighted forState:UIControlStateHighlighted];
    [customLeftBtn setBackgroundImage:leftHighlighted forState:UIControlStateSelected];
    [customLeftBtn.layer setCornerRadius:customLeftBtn.frame.size.width/2];
    [customLeftBtn.layer setMasksToBounds:YES];
    self.controlView.controlView.leftControlBtn = customLeftBtn;
    UIButton *customRightBtn = [[UIButton alloc] initWithFrame:CGRectZero];
    UIImage *rightNormal = [UIImage imageNamed:@"arrow_right_custom" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    UIImage *rightHighlighted = [UIImage imageNamed:@"arrow_right_highlighted_custom" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    [customRightBtn setBackgroundImage:rightNormal forState:UIControlStateNormal];
    [customRightBtn setBackgroundImage:rightHighlighted forState:UIControlStateHighlighted];
    [customRightBtn setBackgroundImage:rightHighlighted forState:UIControlStateSelected];
    [customRightBtn.layer setCornerRadius:customRightBtn.frame.size.width/2];
    [customRightBtn.layer setMasksToBounds:YES];
    self.controlView.controlView.rightControlBtn = customRightBtn;
    
    UIButton *customUpBtn = [[UIButton alloc] initWithFrame:CGRectZero];
    UIImage *upNormal = [UIImage imageNamed:@"arrow_up_custom" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    UIImage *upHighlighted = [UIImage imageNamed:@"arrow_up_highlighted_custom" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    [customUpBtn setBackgroundImage:upNormal forState:UIControlStateNormal];
    [customUpBtn setBackgroundImage:upHighlighted forState:UIControlStateHighlighted];
    [customUpBtn setBackgroundImage:upHighlighted forState:UIControlStateSelected];
    [customUpBtn.layer setCornerRadius:customUpBtn.frame.size.width/2];
    [customUpBtn.layer setMasksToBounds:YES];
    self.controlView.controlView.upControlBtn = customUpBtn;
    
    UIButton *customDownBtn = [[UIButton alloc] initWithFrame:CGRectZero];
    UIImage *downNormal = [UIImage imageNamed:@"arrow_down_custom" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    UIImage *downHighlighted = [UIImage imageNamed:@"arrow_down_highlighted_custom" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    [customDownBtn setBackgroundImage:downNormal forState:UIControlStateNormal];
    [customDownBtn setBackgroundImage:downHighlighted forState:UIControlStateHighlighted];
    [customDownBtn setBackgroundImage:downHighlighted forState:UIControlStateSelected];
    [customDownBtn.layer setCornerRadius:customDownBtn.frame.size.width/2];
    [customDownBtn.layer setMasksToBounds:YES];
    self.controlView.controlView.downControlBtn = customDownBtn;
    self.controlView.controlView.hidden = NO;
    
}

- (void)creatrCustomZoomControll{
    
    UIButton *customZoomIn = [[UIButton alloc] initWithFrame:CGRectZero];
    UIImage *zoomInNormal = [UIImage imageNamed:@"zoom_in_custom" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    UIImage *zoomInHighlighted = [UIImage imageNamed:@"zoom_in_highlighted_custom" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    [customZoomIn setBackgroundImage:zoomInNormal forState:UIControlStateNormal];
    [customZoomIn setBackgroundImage:zoomInHighlighted forState:UIControlStateHighlighted];
    [customZoomIn setBackgroundImage:zoomInHighlighted forState:UIControlStateSelected];
    [customZoomIn.layer setCornerRadius:customZoomIn.frame.size.width/2];
    [customZoomIn.layer setMasksToBounds:YES];
    self.controlView.zoomView.zoomIn = customZoomIn;
    
    UIButton *customZoomOut = [[UIButton alloc] initWithFrame:CGRectZero];
    UIImage *zoomOutNormal = [UIImage imageNamed:@"zoom_out_custom" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    UIImage *zoomOutHighlighted = [UIImage imageNamed:@"zoom_out_highlighted_custom" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    [customZoomOut setBackgroundImage:zoomOutNormal forState:UIControlStateNormal];
    [customZoomOut setBackgroundImage:zoomOutHighlighted forState:UIControlStateHighlighted];
    [customZoomOut setBackgroundImage:zoomOutHighlighted forState:UIControlStateSelected];
    [customZoomOut.layer setCornerRadius:customZoomOut.frame.size.width/2];
    [customZoomOut.layer setMasksToBounds:YES];
    self.controlView.zoomView.zoomOut = customZoomOut;
    self.controlView.zoomView.hidden = NO;
}

- (void)setUpLayoutConstraintCenterButton:(UIButton *)button {
    [button setTranslatesAutoresizingMaskIntoConstraints:NO];
    CGFloat _leftMargin = 4;
    CGFloat _bottomMargin = -60;
    CGFloat _height = 36;
    CGFloat _width = 36;
    NSLayoutConstraint *left = [NSLayoutConstraint constraintWithItem:button attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:self.mapView attribute:NSLayoutAttributeLeft multiplier:1 constant:_leftMargin];
    NSLayoutConstraint *bottom = [NSLayoutConstraint constraintWithItem:button attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:self.mapView attribute:NSLayoutAttributeBottom multiplier:1 constant:_bottomMargin];

    NSLayoutConstraint *height = [NSLayoutConstraint constraintWithItem:button attribute:NSLayoutAttributeHeight relatedBy:NSLayoutRelationEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:_height];
    NSLayoutConstraint *width = [NSLayoutConstraint constraintWithItem:button attribute:NSLayoutAttributeWidth relatedBy:NSLayoutRelationEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:_width];
    [self.mapView addConstraints:@[left, bottom]];
    [button addConstraints:@[height, width]];
}

- (UIButton *)createCustomComapssButton {
    UIButton *customCompassButton = [[UIButton alloc] initWithFrame:CGRectZero];
    [self.mapView addSubview:customCompassButton];
    [self setUpLayoutConstraintCompassButton:customCompassButton];
    UIImage *compassDefaultImage = [UIImage imageNamed:@"custom_compass" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    UIImage *compassNorthUpImage = [UIImage imageNamed:@"custom_compass" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    UIImage *compassPressedImage = [UIImage imageNamed:@"custom_compass" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    [customCompassButton setBackgroundImage:compassDefaultImage forState:UIControlStateNormal];
    [customCompassButton setBackgroundImage:compassPressedImage forState:UIControlStateHighlighted];
    [customCompassButton setBackgroundImage:compassNorthUpImage forState:UIControlStateSelected];
    [customCompassButton.layer setCornerRadius:customCompassButton.frame.size.width/2];
    [customCompassButton.layer setMasksToBounds:YES];
    return customCompassButton;
}

- (void)setUpLayoutConstraintCompassButton:(UIButton *)button {
    [button setTranslatesAutoresizingMaskIntoConstraints:NO];
    CGFloat _leftMargin = 8;
    CGFloat _topMargin = 70;
    CGFloat _height = 40;
    CGFloat _width = 40;

    NSLayoutConstraint *left = [NSLayoutConstraint constraintWithItem:button attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:self.mapView attribute:NSLayoutAttributeLeft multiplier:1 constant:_leftMargin];
    NSLayoutConstraint *top = [NSLayoutConstraint constraintWithItem:button attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:self.mapView attribute:NSLayoutAttributeTop multiplier:1 constant:_topMargin];

    NSLayoutConstraint *height = [NSLayoutConstraint constraintWithItem:button attribute:NSLayoutAttributeHeight relatedBy:NSLayoutRelationEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:_height];
    NSLayoutConstraint *width = [NSLayoutConstraint constraintWithItem:button attribute:NSLayoutAttributeWidth relatedBy:NSLayoutRelationEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:_width];

    [self.mapView addConstraints:@[left, top]];
    [button addConstraints:@[height, width]];
}


@end
