/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "UIMapViewController.h"
#import "UIMapViewController+CenterLocation.h"
#import "UIMapViewController+SegmentedControl.h"
#import "UIView+Toast.h"

@interface UIMapViewController ()

@property (nonatomic) UIView *toast;
@property (nonatomic, strong) NSTimer * toastTimer;

@end

@implementation UIMapViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self setSegmentHidden:YES];
}

- (void)setTitle:(NSString *)title subtitle:(NSString *)subtitle {
    self.navTitle.text = title;
    self.navSubtitle.text = subtitle;
}

- (BOOL)prefersStatusBarHidden {
    return NO;
}

- (void)showSearchView {
    if (!self.searchViewController) {
        UIStoryboard *mainStoryboard = [UIStoryboard storyboardWithName:@"Main" bundle: nil];
        self.searchViewController = (SearchFieldsViewController *)[mainStoryboard instantiateViewControllerWithIdentifier:@"SearchFieldsViewController"];
        //self.searchViewController.naivgationProtocol = self;
        [self slideDownFrameFromNavigationBarForVC:self.searchViewController];
    }
    
    [self addChildViewController:self.searchViewController];
    [self.view addSubview:self.searchViewController.view];
    [self.searchViewController didMoveToParentViewController:self];
    
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)hideSearchView {
    [self.searchViewController willMoveToParentViewController:nil];
    [self.searchViewController.view removeFromSuperview];
    [self.searchViewController removeFromParentViewController];
    self.searchViewController = nil;
}

- (void)slideDownFrameFromNavigationBarForVC:(UIViewController *)viewController {
    CGFloat height = CGRectGetHeight(self.navigationController.navigationBar.frame) + CGRectGetHeight([[UIApplication sharedApplication] statusBarFrame]);
    [viewController.view setFrame:CGRectMake(0, height, CGRectGetWidth(self.view.frame), CGRectGetHeight(self.view.frame) - height)];
}

#pragma mark - Buttons
- (void)clearSegmentedControll {
    [self setSegmentHidden:YES];
}

- (void)createSegmentedControllWithTag:(NSInteger)tag options:(NSArray<NSString *> *)options multiSelect:(BOOL)multiselect select:(NSInteger)select {
    self.multiSelect = multiselect;
    [self setSegmentTag:tag];
    [self createSegments:options];
    [self deselectAll];
    if(select >= 0) {
        [self selectSegment:select selected:YES];
    }
}

#pragma mark - Option Buttton
- (IBAction)option1Changed:(UIButton *)sender {
    [self optionChangedButton:sender Index:2];
}

- (IBAction)option2Changed:(UIButton *)sender {
    [self optionChangedButton:sender Index:1];
}

- (IBAction)option3Chnaged:(UIButton *)sender {
    [self optionChangedButton:sender Index:0];
}

- (void)optionChangedButton:(UIButton *)button Index:(int)index {
    if(self.multiSelect) {
        [self toogleSelectSegment:index];
    } else {
        [self selectSegment:index selected:YES];
    }
    
     if ([self.delegate respondsToSelector:@selector(optionButtonTag:index:)]) {
        [self.delegate optionButtonTag:button.tag index:index];
    }
}

#pragma mark - Internal

- (void)createMenuNavigationItem {
    UIImage *menuImage = [[UIImage imageNamed:@"btn_menu"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal];
    UIBarButtonItem *optionsItem = [[UIBarButtonItem alloc] initWithImage:menuImage
                                                                    style:UIBarButtonItemStylePlain
                                                                   target:self
                                                                   action:@selector(displayOptionsVC:)];
    self.navigationItem.leftBarButtonItem = optionsItem;
}

- (void)addNavigationButtonWithAction:(nullable SEL)action title:(NSString *)title nextAction:(nullable SEL)nextAction {
    UIBarButtonItem *button = [[UIBarButtonItem alloc] initWithTitle:title
                                                                   style:UIBarButtonItemStylePlain
                                                                  target:self
                                                                  action:action];
    self.nextAction = nextAction;
    self.navigationItem.rightBarButtonItem = button;
}

- (void)displayOptionsVC:(UNUSED_PARAM id)sender {
    
}

#pragma mark - Show Toast

- (void)showToast:(NSString *)text {
    if (self.showToastEnable) {
        
        if (self.toastTimer != nil) {
            [self.toastTimer invalidate];
        }
        
        NSTimeInterval interval = [AppDelegateInstance isTesting] ? 5.0 : 1.25;
        self.toastTimer = [NSTimer scheduledTimerWithTimeInterval: interval
                                                           target: self
                                                         selector: @selector(hideToast)
                                                         userInfo: nil
                                                          repeats: NO];
        
        CSToastStyle *style = [self csToastStyle];
        [self hideToast];
        self.toast = [self.view toastViewForMessage:text title:nil image:nil style:style];
        [CSToastManager setSharedStyle:style];
        [CSToastManager setTapToDismissEnabled:YES];
        [CSToastManager setQueueEnabled:NO];
        [self.view showToast:self.toast duration:60.0 position:CSToastPositionBottom completion:nil];
    }
}

- (CSToastStyle *)csToastStyle {
    CSToastStyle *style = [[CSToastStyle alloc] initWithDefaultStyle];
    style.backgroundColor = [UIColor whiteColor];
    style.cornerRadius = 0.0;
    style.messageColor = [UIColor blackColor];
    style.displayShadow = YES;
    style.shadowOffset = CGSizeZero;
    style.shadowOpacity = 0.3;
    return style;
}

- (void)hideToast {
    [self.toast hideToast:self.toast];
    self.toast = nil;
}

@end
