/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "UISearchViewController.h"
#import <TomTomOnlineSDKSearch/TomTomOnlineSDKSearch.h>
#import "LocationManager.h"
#import "Colors.h"
#import "HeaderView.h"

@interface UISearchViewController () <UISearchBarDelegate, UISearchBarDelegate>

@property (nonatomic, assign) SearchMode mode;
@property (nonatomic, weak) IBOutlet UISearchBar *searchBar;
@property (nonatomic, weak) IBOutlet UISegmentedControl *segmentedControl;
@property (nonatomic, assign) NSInteger selectedIndex;
@property (nonatomic, weak) HeaderView *headerView;
@property (nonatomic, weak) SearchViewController *searchViewController;

@end

@implementation UISearchViewController

-(instancetype)initWithMode:(SearchMode)mode {
    self = [super init];
    if(self) {
        self.mode = mode;
        self.selectedIndex = -1;
    }
    return self;
}

- (void)buildHeader:(NSString*)TitleB and:(NSString*) SubtitleB{
    HeaderView *headerView = [[HeaderView alloc] init];
    self.headerView = headerView;
    self.headerView.title.text = TitleB;
    self.headerView.subtitle.text = SubtitleB;
    self.navigationItem.titleView = self.headerView;
    [self updateHeaderViewLayout];
    [self buildBackButton];
}

- (void)updateHeaderViewLayout {
    CGFloat _width = CGRectGetWidth(self.view.frame);
    CGFloat _height = 40.0;

    [self.navigationItem.titleView removeConstraints:[self.navigationItem.titleView constraints]];

    [self.navigationItem.titleView addConstraints:@[[NSLayoutConstraint constraintWithItem:self.navigationItem.titleView attribute:NSLayoutAttributeHeight relatedBy:NSLayoutRelationEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:_height], [NSLayoutConstraint constraintWithItem:self.navigationItem.titleView attribute:NSLayoutAttributeWidth relatedBy:NSLayoutRelationEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:_width]]];
    [self.navigationItem.titleView setFrame:CGRectMake(0, 0, _width, _height)];
    [self.navigationItem.titleView setNeedsUpdateConstraints];
}

- (void)buildBackButton {
    UIImage *menuImage = [[UIImage imageNamed:@"BtnPixelDot"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal];
    UIBarButtonItem *optionsItem = [[UIBarButtonItem alloc] initWithImage:menuImage
                                                                    style:UIBarButtonItemStylePlain
                                                                   target:self.navigationController
                                                                   action:@selector(popViewControllerAnimated:)];
    self.navigationItem.leftBarButtonItem = optionsItem;
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator {
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];
    [coordinator animateAlongsideTransition:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull UNUSED_PARAM context) {
        self.headerView.hidden = YES;
    } completion:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull UNUSED_PARAM context) {
        [self updateHeaderViewLayout];
        self.headerView.hidden = NO;
    }];
}

- (void)setSearchMode:(SearchMode)mode {
    self.mode = mode;
    [self setupSearchViewController];
}

- (void)setupSearchViewController {
    self.searchBar.hidden = NO;
    self.segmentedControl.hidden = YES;
    [self.searchBar setShowsScopeBar:YES];
    if(self.mode == SearchModeAddress) {
        [self buildHeader:@"Title" and:@"Subtitle"];
        [self buildHeader:@"Address search" and:@"Search for address or POI"];
        [self setupSearchBar];
        self.searchBar.scopeButtonTitles = @[@"GLOBAL", @"NEAR ME"];
        self.searchBar.selectedScopeButtonIndex = 0;
    } else if (self.mode == SearchModeCategory) {
        self.searchBar.hidden = YES;
        self.segmentedControl.hidden = NO;
        [self buildHeader:@"Category search" and:@"Search for POIs in one of categories"];
        self.segmentedControl.selectedSegmentIndex = -1;
        [self.segmentedControl addTarget:self action:@selector(searchCategory:) forControlEvents:UIControlEventValueChanged];
    } else if(self.mode == SearchModeFuzzy) {
        [self buildHeader:@"Max fuzziness levels"and: @"It enables smart search in case of typos in query"];
        [self setupSearchBar];
        self.searchBar.scopeButtonTitles = @[@"Max 1", @"Max 2",@"Max 3"];
        self.searchBar.selectedScopeButtonIndex = 0;
    } else if (self.mode == SearchModeAutocomplete) {
        [self buildHeader:@"Typeahead parameter" and:@"Enable typeahead suggestions"];
        [self setupSearchBar];
    } else if (self.mode == SearchModeLanguage) {
        [self buildHeader:@"Language selector" and:@"Choose the language for search results"];
        [self setupSearchBar];
        self.searchBar.scopeButtonTitles = @[@"🇬🇧 EN", @"🇩🇪 DE", @"🇪🇸 ES", @"🇫🇷 FR"];
        self.searchBar.selectedScopeButtonIndex = 0;
    }
}

- (void)setupSearchBar {
    self.searchBar.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    self.searchBar.backgroundColor = [UIColor whiteColor];
    self.searchBar.clipsToBounds = YES;
    [self.searchBar sizeToFit];
    self.searchBar.delegate = self;
    self.searchBar.tintColor = TTColorGreen;
    self.searchBar.searchBarStyle = UISearchBarStyleDefault;
    self.searchBar.barTintColor = TTColorWhite;
    self.searchBar.backgroundColor = TTColorWhite;
    self.searchBar.translucent = NO;
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    self.searchViewController = [self.childViewControllers firstObject];
    self.searchViewController.searchBar = self.searchBar;
    [self setupSearchViewController];
}

- (void)searchCategory:(UISegmentedControl *)sender {
    self.selectedIndex = sender.selectedSegmentIndex;
    NSString *query = [sender titleForSegmentAtIndex:(NSUInteger)self.selectedIndex];
    [self.searchViewController performCategorySearchWithQuery:query];
}

- (void)doSearch {
    [self.searchBar resignFirstResponder];
    NSString *query = self.searchBar.text;
    if(query.length == 0) {
        return;
    }

    if (self.mode == SearchModeAutocomplete) {
        [self.searchViewController performAutocompleteSearchWithQuery:query];
        return;
    }
    
    if (self.mode == SearchModeAddress) {
        if(self.searchBar.selectedScopeButtonIndex == 0) {
            [self.searchViewController performAddressSearchWithQuery:query];
        } else {
            CLLocationCoordinate2D myLocation =[[[LocationManager sharedInstance] location] coordinate];
            [self.searchViewController performAddressSearchWithQuery:query position:myLocation];
        }
    
    } else if (self.mode == SearchModeFuzzy) {
        int maxFuzzy = (int)self.searchBar.selectedScopeButtonIndex+1;
        CLLocationCoordinate2D myLocation = [[[LocationManager sharedInstance] location] coordinate];
        [self.searchViewController performAddressSearchWithQuery:query fuzzyMin:1 fuzzyMax:maxFuzzy position: myLocation];
        
    } else if (self.mode == SearchModeLanguage) {
        TTLanguage languageType = TTLanguageEnglishGB;
        switch (self.searchBar.selectedScopeButtonIndex) {
            case 1: // German
                languageType = TTLanguageGerman;
                break;
            case 2: // Spanish ES
                languageType = TTLanguageSpanishES;
                break;
            case 3: // French FR
                languageType = TTLanguageFrenchFR;
                break;
        }
        
        [self.searchViewController performAddressSearchWithQuery:query language:languageType];
    }
}

#pragma mark UISearchBarDelegate
- (void)searchBarSearchButtonClicked:(UNUSED_PARAM UISearchBar *)searchBar {
    [self doSearch];
}

- (void)searchBar:(UNUSED_PARAM UISearchBar *)searchBar textDidChange:(UNUSED_PARAM  NSString *)searchText {
    if (self.mode == SearchModeAutocomplete) {
        NSString *query = self.searchBar.text;
        if(query.length == 0) {
            return;
        }
        [self.searchViewController performAutocompleteSearchWithQuery:query];
        [[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:YES];
    }
}

#pragma mark UISearchBarDelegate
- (void)searchBar:(UNUSED_PARAM UISearchBar *)searchBar selectedScopeButtonIndexDidChange:(UNUSED_PARAM NSInteger)selectedScope {
    [self doSearch];
}

@end
